﻿using TypographyContracts.BindingModels;
using TypographyContracts.SearchModels;
using TypographyContracts.StoragesContracts;
using TypographyContracts.ViewModels;
using TypographyDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace TypographyDatabaseImplements.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new TypographyDatabase();
            return context.Orders
                .Include(x => x.Printed)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .FirstOrDefault(x =>
                (model.Status == null || model.Status != null && model.Status.Equals(x.Status)) &&
                (model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId) ||
                (model.Id.HasValue && x.Id == model.Id))?
                .GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue && model.Status == null)
            {
                return new();
            }
            using var context = new TypographyDatabase();
            return context.Orders
                .Where(x =>
                x.Id == model.Id ||
                model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo ||
                x.ClientId == model.ClientId ||
                model.Status.Equals(x.Status))
                .Include(x => x.Printed)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new TypographyDatabase();
            return context.Orders
                .Include(x => x.Printed)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Select(x => x.GetViewModel).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new TypographyDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                .Include(x => x.Printed)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new TypographyDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders.Include(x => x.Printed).Include(x => x.Client).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new TypographyDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}